/* adc_mcp9700

   Folyamatos módban mérünk az ADC-vel, a konverziók végén
   programmegszakítást kérünk, s az ADC megszakításkiszolgáló
   eljárásban olvassuk ki az eredményt.
   Az ADC-vel itt az A4 bemenetre kapcsolt MVP9700 analóg
   hőmérő jelét mérjükm az 1.1 V-os belső referenciához viszonyítva.
*/

volatile long adc_sum = 0; // analog value
volatile long adc_val = 0; // ADC 1100 mérés összege
uint16_t adc_cnt = 1100;   // ADC számláló

void setup() {
  Serial.begin(115200);
  //--- ADC indítás megszakításos módban ---
  DIDR0 = 0x3F;            // digital inputs disabled
  ADMUX = _BV(REFS0) | _BV(REFS1) | _BV(MUX2); // A4 csatorna, 1.1 V ref
  ADCSRA = _BV(ADEN) |     // ADC enable
           _BV(ADATE) |    // auto trigger enable
           _BV(ADIE) |     // interrupt enable
           _BV(ADPS2);     // prescaler = 16  (azaz 2^4)
  ADCSRB = 0x00;           // AD channels MUX off, free running mode
  ADCSRA |= _BV(ADSC);     // Konverzió indítása
  sei();                   // megszakítás globális engedélyezése
}

void loop() {
  delay(1000);
  long mv = adc_val / 1024;
  float tempC = (mv - 500) / 10.0;
  Serial.print("temp = ");
  Serial.print(tempC, 1);
  Serial.println(" °C");
}

//--- 'ADC kész' megszakítás kiszolgálása ---
ISR(ADC_vect) {
  adc_sum += ADCW;         // 1100 mérést összegzünk
  if (--adc_cnt == 0) {
    adc_val = adc_sum;
    adc_sum = 0;
    adc_cnt = 1100;
  }
}
