
/* newping_7seg
 *  
 *  Távolságmérés HC-SR04 ultrahangos szenzorral, a NewPing
 *  programkönyvtár használatával. A távolságot 250 ms-onként
 *  mérjük, s a Multifunkciós kártya 7-szegmenses kijelzőjén
 *  centiméterekben kiírjuk az eredményt.
 */
#include <NewPing.h>

/* HC-SR04 paraméterek */
#define TRIGGER_PIN  5
#define ECHO_PIN     6
#define MAX_DISTANCE 200

/* MF kártya kijelző vezérlés */

/* Szegmensrajzolat a 0-9 számjegyekhez */
/* A kijelzőt vezérlő lábak definiálása */
#define LATCH_DIO 4
#define CLK_DIO   7
#define DATA_DIO  8
const byte SEGMENT_MAP[] = {0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0X80, 0X90};
/* Az 1-4 számjegyek kiválasztása */
const byte SEGMENT_SELECT[] = {0xF1, 0xF2, 0xF4, 0xF8};
volatile byte segment_data[] = {0, 0, 0, 0}; // adattároló buffer
byte idx = 0;                             // Kijelzés számlálója
long nexttime;                            // Időbélyeg mérés indításhoz

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);

void setup() {
  /* Vezérlő lábak konfigurálása */
  pinMode(LATCH_DIO, OUTPUT);
  pinMode(CLK_DIO, OUTPUT);
  pinMode(DATA_DIO, OUTPUT);
  nexttime = millis();
}

void loop() {
  if ( millis() > nexttime) {
    int distance = sonar.ping_cm();
    segment_data[0] = 0xFF;
    segment_data[1] = SEGMENT_MAP[(distance / 100) % 10];
    segment_data[2] = SEGMENT_MAP[(distance  / 10) % 10];
    segment_data[3] = SEGMENT_MAP[distance  % 10];
    nexttime += 250;
  }
  /* Mérési szünetekben pörgetjük a kijelzést */
  refresh_display();
}

/* A soron következő számjegy megjelenítése */
void refresh_display() {
  byte i;
  i = idx++ & 0x3;
  digitalWrite(LATCH_DIO, LOW);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, segment_data[i]);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, SEGMENT_SELECT[i] );
  digitalWrite(LATCH_DIO, HIGH);
}
